import unittest
from unittest.mock import Mock

from AnyQt.QtWidgets import QComboBox, QCheckBox, QSpinBox, QLineEdit

from orangewidget.tests.base import GuiTest
from orangewidget.utils.visual_settings_dlg import SettingsDialog, FontList


class TestSettingsDialog(GuiTest):
    def setUp(self):
        self.defaults = {
            "Box": {"Items": {
                "P1": (["Foo", "Bar", "Baz"], "Bar"),
                "P2": (range(3, 10, 2), 5),
                "P3": (None, True),
                "P4": (None, "Foo Bar"),
                "P5": (FontList([".Foo", ".Bar"]), ".Foo"),
            }}
        }
        self.dlg = SettingsDialog(None, self.defaults)

    @property
    def dialog_controls(self):
        return self.dlg._SettingsDialog__controls

    def test_initialize(self):
        controls = self.dialog_controls
        self.assertEqual(len(controls), len(self.defaults["Box"]["Items"]))
        self.assertIsInstance(controls[("Box", "Items", "P1")][0], QComboBox)
        self.assertIsInstance(controls[("Box", "Items", "P2")][0], QSpinBox)
        self.assertIsInstance(controls[("Box", "Items", "P3")][0], QCheckBox)
        self.assertIsInstance(controls[("Box", "Items", "P4")][0], QLineEdit)
        self.assertIsInstance(controls[("Box", "Items", "P5")][0], QComboBox)

    def test_changed_settings(self):
        self.dialog_controls[("Box", "Items", "P1")][0].setCurrentText("Foo")
        self.dialog_controls[("Box", "Items", "P2")][0].setValue(7)
        self.dialog_controls[("Box", "Items", "P3")][0].setChecked(False)
        self.dialog_controls[("Box", "Items", "P4")][0].setText("Foo Baz")
        self.dialog_controls[("Box", "Items", "P5")][0].setCurrentIndex(1)
        changed = {("Box", "Items", "P1"): "Foo",
                   ("Box", "Items", "P2"): 7,
                   ("Box", "Items", "P3"): False,
                   ("Box", "Items", "P4"): "Foo Baz",
                   ("Box", "Items", "P5"): ".Bar"}
        self.assertDictEqual(self.dlg.changed_settings, changed)

    def test_reset(self):
        ctrls = self.dialog_controls
        ctrls[("Box", "Items", "P1")][0].setCurrentText("Foo")
        ctrls[("Box", "Items", "P2")][0].setValue(7)
        ctrls[("Box", "Items", "P3")][0].setChecked(False)
        ctrls[("Box", "Items", "P4")][0].setText("Foo Baz")
        self.dialog_controls[("Box", "Items", "P5")][0].setCurrentIndex(1)

        self.dlg._SettingsDialog__reset()
        self.assertDictEqual(self.dlg.changed_settings, {})
        self.assertEqual(ctrls[("Box", "Items", "P1")][0].currentText(), "Bar")
        self.assertEqual(ctrls[("Box", "Items", "P2")][0].value(), 5)
        self.assertTrue(ctrls[("Box", "Items", "P3")][0].isChecked())
        self.assertEqual(ctrls[("Box", "Items", "P4")][0].text(), "Foo Bar")
        self.assertEqual(ctrls[("Box", "Items", "P5")][0].currentText(), "Foo")

    def test_setting_changed(self):
        handler = Mock()
        self.dlg.setting_changed.connect(handler)
        self.dialog_controls[("Box", "Items", "P1")][0].setCurrentText("Foo")
        handler.assert_called_with(('Box', 'Items', 'P1'), "Foo")
        self.dialog_controls[("Box", "Items", "P2")][0].setValue(7)
        handler.assert_called_with(('Box', 'Items', 'P2'), 7)
        self.dialog_controls[("Box", "Items", "P3")][0].setChecked(False)
        handler.assert_called_with(('Box', 'Items', 'P3'), False)
        self.dialog_controls[("Box", "Items", "P4")][0].setText("Foo Baz")
        handler.assert_called_with(('Box', 'Items', 'P4'), "Foo Baz")
        self.dialog_controls[("Box", "Items", "P5")][0].setCurrentIndex(1)
        handler.assert_called_with(('Box', 'Items', 'P5'), ".Bar")

    def test_apply_settings(self):
        changed = [(("Box", "Items", "P1"), "Foo"),
                   (("Box", "Items", "P2"), 7),
                   (("Box", "Items", "P3"), False),
                   (("Box", "Items", "P4"), "Foo Baz"),
                   (("Box", "Items", "P5"), ".Bar")]
        self.dlg.apply_settings(changed)
        ctrls = self.dialog_controls
        self.assertEqual(ctrls[("Box", "Items", "P1")][0].currentText(), "Foo")
        self.assertEqual(ctrls[("Box", "Items", "P2")][0].value(), 7)
        self.assertFalse(ctrls[("Box", "Items", "P3")][0].isChecked())
        self.assertEqual(ctrls[("Box", "Items", "P4")][0].text(), "Foo Baz")
        self.assertEqual(ctrls[("Box", "Items", "P5")][0].currentText(), "Bar")
        self.assertDictEqual(self.dlg.changed_settings,
                             {k: v for k, v in changed})


if __name__ == '__main__':
    unittest.main()



import enum
import inspect
from typing import Union

import sys
import warnings
from operator import attrgetter

from AnyQt.QtCore import QObject


def progress_bar_milestones(count, iterations=100):
    return set([int(i*count/float(iterations)) for i in range(iterations)])


_NOTSET = object()


def deepgetattr(obj, attr, default=_NOTSET):
    """Works exactly like getattr(), except that attr can be a nested attribute
    (e.g. "attr1.attr2.attr3").
    """
    try:
        return attrgetter(attr)(obj)
    except AttributeError:
        if default is _NOTSET:
            raise
        return default


def getdeepattr(obj, attr, *arg, **kwarg):
    if isinstance(obj, dict):
        return obj.get(attr)
    return deepgetattr(obj, attr, *arg, **kwarg)


def to_html(str):
    return str.replace("<=", "&#8804;").replace(">=", "&#8805;").\
        replace("<", "&#60;").replace(">", "&#62;").replace("=\\=", "&#8800;")

getHtmlCompatibleString = to_html


def dumpObjectTree(obj, _indent=0):
    """
    Dumps Qt QObject tree. Aids in debugging internals.
    See also: QObject.dumpObjectTree()
    """
    assert isinstance(obj, QObject)
    print('{indent}{type} "{name}"'.format(indent=' ' * (_indent * 4),
                                           type=type(obj).__name__,
                                           name=obj.objectName()),
          file=sys.stderr)
    for child in obj.children():
        dumpObjectTree(child, _indent + 1)


def getmembers(obj, predicate=None):
    """Return all the members of an object in a list of (name, value) pairs sorted by name.

    Behaves like inspect.getmembers. If a type object is passed as a predicate,
    only members of that type are returned.
    """

    if isinstance(predicate, type):
        def mypredicate(x):
            return isinstance(x, predicate)
    else:
        mypredicate = predicate
    return inspect.getmembers(obj, mypredicate)


class DeprecatedSignal:
    def __init__(self, actual_signal, *args,
                 warning_text='Deprecated', emit_callback=None, **kwargs):
        self.signal = actual_signal
        self.warning_text = warning_text
        self.emit_callback = emit_callback

    def emit(self, *args, **kwargs):
        warnings.warn(
            self.warning_text,
            DeprecationWarning, stacklevel=2
        )
        if self.emit_callback:
            self.emit_callback(*args, **kwargs)
        return self.signal.emit(*args, **kwargs)

    def __getattr__(self, item):
        return self.__signal.item


def enum_as_int(value: Union[int, enum.Enum]) -> int:
    """
    Return a `enum.Enum` value as an `int.

    This is function intended for extracting underlying Qt5/6 enum
    values specifically with PyQt6 where most Qt enums are represented
    with `enum.Enum` and lose their numerical value.

    >>> from PyQt6.QtCore import Qt
    >>> enum_as_int(Qt.Alignment.AlignLeft)
    1
    """
    if isinstance(value, enum.Enum):
        return int(value.value)
    else:
        return int(value)



from AnyQt.QtWidgets import (
    QPushButton, QAbstractButton, QFocusFrame, QStyle, QStylePainter,
    QStyleOptionButton
)
from AnyQt.QtGui import QPalette, QIcon
from AnyQt.QtCore import Qt, QSize, QEvent


class VariableTextPushButton(QPushButton):
    """
    QPushButton subclass with an sizeHint method to better support settable
    variable width button text.

    Use this class instead of the QPushButton when the button will
    switch the text dynamically while displayed.
    """
    def __init__(self, *args, textChoiceList=[], **kwargs):
        super().__init__(*args, **kwargs)
        self.__textChoiceList = list(textChoiceList)

    def setTextChoiceList(self, textList):
        """
        Set the list of all `text` string to use for size hinting.

        Parameters
        ----------
        textList : List[str]
            A list of all different `text` properties that will/can be set on
            the push button. This list is used to derive a suitable sizeHint
            for the widget.
        """
        self.__textChoiceList = textList
        self.updateGeometry()

    def sizeHint(self):
        """
        Reimplemented from `QPushButton.sizeHint`.

        Returns
        -------
        sh : QSize
        """
        sh = super().sizeHint()
        option = QStyleOptionButton()
        self.initStyleOption(option)
        style = self.style()
        fm = option.fontMetrics
        if option.iconSize.isValid():
            icsize = option.iconSize
            icsize.setWidth(icsize.width() + 4)
        else:
            icsize = QSize()

        for text in self.__textChoiceList:
            option.text = text
            size = fm.size(Qt.TextShowMnemonic, text)

            if not icsize.isNull():
                size.setWidth(size.width() + icsize.width())
                size.setHeight(max(size.height(), icsize.height()))

            sh = sh.expandedTo(
                style.sizeFromContents(QStyle.CT_PushButton, option,
                                       size, self))
        return sh


class SimpleButton(QAbstractButton):
    """
    A simple icon button widget.
    """
    def __init__(self, parent=None, **kwargs):
        super().__init__(parent, **kwargs)
        self.__focusframe = None

    def focusInEvent(self, event):
        # reimplemented
        event.accept()
        if self.__focusframe is None:
            self.__focusframe = QFocusFrame(self)
            self.__focusframe.setWidget(self)
            palette = self.palette()
            palette.setColor(QPalette.WindowText,
                             palette.color(QPalette.Highlight))
            self.__focusframe.setPalette(palette)

    def focusOutEvent(self, event):
        # reimplemented
        event.accept()
        if self.__focusframe is not None:
            self.__focusframe.hide()
            self.__focusframe.deleteLater()
            self.__focusframe = None

    def event(self, event):
        if event.type() == QEvent.Enter or event.type() == QEvent.Leave:
            self.update()
        return super().event(event)

    def sizeHint(self):
        # reimplemented
        self.ensurePolished()
        iconsize = self.iconSize()
        icon = self.icon()
        if not icon.isNull():
            iconsize = icon.actualSize(iconsize)
        return iconsize

    def minimumSizeHint(self):
        # reimplemented
        return self.sizeHint()

    def paintEvent(self, event):
        painter = QStylePainter(self)
        option = QStyleOptionButton()
        option.initFrom(self)
        option.text = ""
        option.icon = self.icon()
        option.iconSize = self.iconSize()
        option.features = QStyleOptionButton.Flat
        if self.isDown():
            option.state |= QStyle.State_Sunken
            painter.drawPrimitive(QStyle.PE_PanelButtonBevel, option)

        if not option.icon.isNull():
            if option.state & QStyle.State_Active:
                mode = (QIcon.Active if option.state & QStyle.State_MouseOver
                        else QIcon.Normal)
            else:
                mode = QIcon.Disabled
            if self.isChecked():
                state = QIcon.On
            else:
                state = QIcon.Off
            option.icon.paint(painter, option.rect, Qt.AlignCenter, mode, state)
